
##################### Functions for functional data manipulation   #####################

# Convert from funData list to funData
convert_funData_list_to_funData <- function(list) {
  grid <- getArgvals(list[[1]])
  
  f <- function(ii) {
    getX(list[[ii]])
  }
  mat <- t(sapply(1:length(list), f))
  funData_object <- funData(grid, mat)
  return(funData_object)
}

# Convert from funData list to fd list
convert_funData_to_fd_list <- function(funData_list,domain=c(0,1)){
  
  f<-function (ii){
    convert_funData_to_fd(funData_list[[ii]],domain)
  }
  fdlist<-lapply(1:length(funData_list),f)
  return(fdlist)
}

# Convert from funData to fd
convert_funData_to_fd <- function(funDataObj,domain=c(0,1)){
  x <- getArgvals(funDataObj)[[1]]
  X <- getX(funDataObj)
  basis <- create.bspline.basis(domain,100)
  pen <- fdPar(basis,2,10^(-10))
  fd_object <- smooth.basis(x,t(X),pen)$fd
  return(fd_object)
}

# Convert from fd_list to fd
convert_fd_list_to_fd <- function(list){
  P=1000
  grid = seq( 0, 1, length.out = P )
  n_obs<-length(list)
  mat<-matrix(0,P,n_obs)
  f <- function(ii) {
    eval.fd(grid, list[[ii]])
  }
  mat<-sapply(1:n_obs,f)
  basis <- create.bspline.basis(c(0,1),100)
  pen <- fdPar(basis,2,10^(-10))
  fd<-smooth.basis(grid,mat,pen)$fd
  return(fd)
}

# Convert fd in funData
convert_fd_to_funData <- function(fd) {
  P = 1000
  grid = seq(0, 1, length.out = P)
  X <- eval.fd(grid, fd)
  funData1 <- funData(grid, t(X))
  
  return(funData1)
}

# Convert  fd list in funData list
convert_fd_list_to_FunData_list <- function(list){
  P=1000
  grid = seq( 0, 1, length.out = P )
  fda_list_FunData<-list
  f <- function(ii) {
    x <- t(eval.fd(grid, list[[ii]]))
    funData(grid, x)
  }
  fda_list_FunData<-lapply(1:length(list),f)
  return(fda_list_FunData)
}

# Standardize fd
standardize_fda_list <- function(fda_list) {
  fda_list_z <- fda_list
  mean_fd <- vector("list", length = length(fda_list))
  names(mean_fd) <- names(fda_list)
  sd_fd <- vector("list", length = length(fda_list))
  names(sd_fd) <- names(fda_list)
  sd_fd_inv <- vector("list", length = length(fda_list))
  
  names(sd_fd_inv) <- names(fda_list)
  for (ii in 1:length(fda_list)) {
    centered_fd <- center.fd(fda_list[[ii]])
    
    std_dev <- std.fd(fda_list[[ii]])
    sd_fd[[ii]] <- std_dev
    std_dev_inv <- std_dev ^ (-1)
    sd_fd_inv[[ii]] <- std_dev_inv
    mean_fd[[ii]] <- mean.fd(fda_list[[ii]])
    fda_list_z[[ii]] <- centered_fd * std_dev_inv
  }
  
  out <- list(fda_list_z = fda_list_z,
              sd_fd = sd_fd,
              mean_fd = mean_fd)
  return(out)
}

# Standardized fd_given mean and sd
standardize_fda_list_given_means_sds <- function(fda_list, means, sds) {
  fda_list_z <- fda_list
  n_obs <- length(fda_list[[1]]$fdnames$reps)
  f <- function(jj, ii) {
    fda_list[[ii]][jj] - means[[ii]]
  }
  for (ii in 1:length(fda_list)) {
    centered_fd_list <- Map(f, 1:n_obs, ii)
    centered_fd <- convert_fd_list_to_fd(centered_fd_list)
    
    std_dev_inv <- sds[[ii]] ^ (-1)
    
    
    fda_list_z[[ii]] <- centered_fd * std_dev_inv
  }
  
  return(fda_list_z)
}
