
#################### Functions for the simulation  #####################

# Correlation -------------------------------------------------------------

CORR <- function(TIPI,P,time_grid_1 ,kappa) {

  n_var <- length(TIPI)
  cov.pars <- list()
  for (ii in 1:n_var) {
    cov.pars[[ii]] <- c(1, 1)
  }
  
  Type <- list()
  length(Type) <- n_var
  for (ii in 1:n_var) {
    names(Type)[ii] <- paste("X", ii)
    Type[[ii]] <- TIPI[[ii]]
  }
  COV <- Type
  
  for (kk in 1:length(Type)) {
    if (Type[[kk]] == "Bessel") {
      time_grid_b <- seq(0, 4, length.out = P)
      cov <-
        besselJ(time_grid_b, kappa[kk])#bessel function first kind
      covmat <- array(0, dim = c(1, P, P))
      
      for (ii in 1:P) {
        covmat[1, ii, ii:P] <- cov[1:(P - ii + 1)]
        covmat[1, P - ii + 1, 1:(P - ii + 1)] <-
          rev(cov[1:(P - ii + 1)])
      }
      
    }
    else{
      cov <- cov.spatial(time_grid,
                         cov.model = Type[[kk]],
                         cov.pars[[kk]],
                         kappa[kk])
      covmat <- array(0, dim = c(1, P, P))
      
      for (ii in 1:P) {
        covmat[1, ii, ii:P] <- cov[1:(P - ii + 1)]
        covmat[1, P - ii + 1, 1:(P - ii + 1)] <-
          rev(cov[1:(P - ii + 1)])
      }
    }
    cov_funData <- funData(list(time_grid_1, time_grid_1), covmat)
    
    COV[[kk]] <- list(covmat = covmat,
                      cov_funData = cov_funData)
    
  }
  
  return(COV)
}

# Spectral decomposition single correlation------------------------------------------------

Sdecomposition_single <- function(covar,P,time_grid,n_auto) {
  
  
  w <- 1.437399 / P
  decomposition <- list()
  n_var <- length(covar)
  for (ii in 1:n_var) {
    X <- covar[[ii]][[1]][1, , ]
    decomposition[[ii]] <- eigen(X)
    names(decomposition)[ii] <- names(covar)[ii]
  }
  
  eigenfunctions_funData <- list()
  for (kk in 1:n_var) {
    eigenfunctions_funData[[kk]] <- list()
    for (ii in 1:n_auto) {
      a <- matrix(0, 1, P)
      a[1, ] <- decomposition[[kk]]$vectors[,ii]
      eigenfunctions_funData[[kk]][[ii]] <-
        funData(time_grid, a) * w ^ (-1 / 2)
    }
    names(eigenfunctions_funData)[kk] <- names(covar)[kk]
  }
  
  for (ii in 1:n_var) {
    decomposition[[ii]][[1]] <-
      replace(decomposition[[ii]][[1]], seq(1, P), decomposition[[ii]][[1]] *
                w)
    
  }
  
 # Fitted correlations
  fittedcov <- list()
  for (kk in 1:n_var) {
    cov <- 0
    for (ii in 1:n_auto) {
      cov <-
        cov + decomposition[[kk]]$values[ii] *
        tensorProduct(eigenfunctions_funData[[kk]][[ii]], eigenfunctions_funData[[kk]][[ii]])
      
    }
    fittedcov[[kk]] <- cov
    names(fittedcov)[kk] <- names(covar)[kk]
  }
  
  
  # Cross-correlations 
  
  
  croscor_funData <- list()
  gain <- seq(0, n_auto)
  for (jj in 1:n_auto) {
    
    sum <- 0
    for (ii in 1:n_var) {
      sum <- sum + decomposition[[ii]][[1]][jj]
    }
    gain[jj] <- sum / (n_var)
  }
  for (ll in 1:n_var) {
    
    croscor_funData[[ll]] <- list()
    for (kk in 1:n_var) {
      sum <- 0
      if (ll == kk)
        sum <- covar[[ll]][[2]]
      
      else
        
        for (jj in 1:n_auto) {
          sum <-
            sum + gain[jj] * (tensorProduct(eigenfunctions_funData[[ll]][[jj]],
                                            eigenfunctions_funData[[kk]][[jj]]))/3
            
          
        }
      croscor_funData[[ll]][[kk]] <- sum
    }
    
  }
  
  croscormat<-list()
  for (kk in 1:n_var) {
    croscormat[[kk]]<-list()
    for (ll in 1:n_var) {
      croscormat[[kk]][[ll]]<-getX(croscor_funData[[kk]][[ll]])[1,,]
    }
  }
  
  # Eigenfunctions
  
  Eigenfunctions_list <- list()
  for (ii in 1:n_auto) {
    Eigenfunctions_list[[ii]] <- list()
    for (jj in 1:n_var) {
      Eigenfunctions_list[[ii]][[jj]] <- eigenfunctions_funData[[jj]][[ii]]
    }
  }
  Eigenfunctions <- list()
  for (ii in 1:n_auto) {
    Eigenfunctions[[ii]] <-
      convert_funData_list_to_funData(Eigenfunctions_list[[ii]])
  }
  out<-list(decomposition=decomposition,
            eigenfunctions_funData=eigenfunctions_funData,
            fittedcov=fittedcov,
            croscor=list(croscor_funData=croscor_funData,croscormat=croscormat),
            Eigenfunctions=Eigenfunctions
  )
  return(out)
}

# Spectral decomposition total -----------------------------------------

Sdecomposition_total<-function(dec, P, time_grid,n_auto){
  w <- 1 / P
  hbind <- list()
  n_var <- length(dec$decomposition)
  for (jj in 1:n_var) {
    a <- dec$croscor$croscormat[[jj]][[1]]
    if (n_var != 1)
      for (ii in 2:n_var) {
        a <- cbind(a, dec$croscor$croscormat[[jj]][[ii]])
      }
    
    hbind[[jj]] <- a
  }
  bigmatrix <- hbind[[1]]
  if (n_var != 1)
    for (ii in 2:n_var) {
      bigmatrix <- rbind(bigmatrix, hbind[[ii]])
    }
  dim(bigmatrix)
  
  e <- eigen(bigmatrix)
  
  seq_list <- list()
  for (ii in 1:n_var) {
    seq_list[[ii]] <- seq(ii * P - P + 1, ii * P)
  }
  
  eigenfunctions_list <- list()
  eigenfunctions_funData <- list()
  
  for (ii in 1:n_auto) {
    eigenfunctions_list[[ii]] <- list()
    for (jj in 1:n_var) {
      a <- matrix(0, 1, P)
      a[1, ] <- e$vectors[seq_list[[jj]], ii]
      
      eigenfunctions_list[[ii]][[jj]] <- funData(time_grid, a) * w ^ (-1 /
                                                                        2)
    }
    eigenfunctions_funData[[ii]] <-
      convert_funData_list_to_funData(eigenfunctions_list[[ii]])
    
  }
  
  e$values<- e$values*w
  
  fittedcov <- list()
  for (kk in 1:n_var) {
    fittedcov[[kk]] <- list()
    for (ll in 1:n_var) {
      cov <- 0
      for (ii in 1:n_auto) {
        cov <-
          cov + e$values[ii] *
          tensorProduct(
            extractObs(eigenfunctions_funData[[ii]], kk),
            extractObs(eigenfunctions_funData[[ii]], ll)
          )
        
      }
      fittedcov[[kk]][[ll]] <- cov
      
    }
  }
  
  out<-list(eigenvalues_tot=e$values[1:n_auto],
            eigenfunctions_tot_funData=eigenfunctions_funData,
            fittedcor_tot=fittedcov
  )
  return(out)
}


# Mean calculation --------------------------------------------------------

me_va <- function(medie, deviazioni, f, sca) {
  seg <- sapply(time_grid, f)
  
  m <- numeric()
  if (length(medie) != 1) {
    for (ii in 1:P) {
      sum <- 0
      for (jj in 1:length(medie)) {
        sum <-
          sum + sca * dnorm(time_grid[ii] ,
                            mean = medie[jj],
                            sd = deviazioni[jj],
                            log = FALSE)
      }
      m[ii] = sum + seg[ii]
      
    }
  }
  else
    m <- seg
  
  m
  
}



# Data generation ---------------------------------------------------------

single_simulation_run_shift <-
  function(dec_X_tot,
           dec_Y_tot,
           n_auto_X,
           n_auto_Y,
           diag_PLS,
           n_obs,
           sigma_M_diag,
           V_X,
           V_Y,
           M_X,
           M_Y,
           grid_obs,
           var_x,
           var_y,
           time_grid,
           P,
           beta_delta_fd_sum) {
    
    
    PLS <- diag(diag_PLS, n_auto_X, n_auto_Y)
    
    # Compute scores and errors
    n_var_X<-length(dec_X_tot$fittedcor_tot[[1]])
    sigma_csi_diag<-dec_X_tot$eigenvalues_tot
    sigma_tot<-diag(c(sigma_csi_diag,sigma_M_diag))
    csi_epsilon<-mvrnorm(n_obs,rep(0,n_auto_X+n_auto_Y),sigma_tot)
    csi_X<-csi_epsilon[,1:n_auto_X]
    epsilon<-csi_epsilon[,(n_auto_X+1):ncol(csi_epsilon)]
    csi_Y<-csi_X%*%PLS+epsilon
    
    f <- function(kk) {
      X_Z <- list()
      for (jj in 1:n_var_X) {
        X_Z[[jj]] <-
          Reduce("+", Map('*', csi_X[kk,] , extractObs(
            multiFunData(dec_X_tot$eigenfunctions_tot_funData), jj
          )))
      }
      return(X_Z)
    }
    X_Z<-lapply(1:n_obs,f)
    
    f<-function (jj,kk) {
      X_Z[[jj]][[kk]]
    }
    X_Z2<-list()
    X_Z_funData<-list()
    for (ii in 1:n_var_X) {
      X_Z2[[ii]]<-Map(f,1:n_obs,ii)
      X_Z_funData[[ii]]<-convert_funData_list_to_funData( X_Z2[[ii]])
    }
    
    f <- function(kk) {
      Reduce("+", Map('*',csi_Y[kk, ] , extractObs(multiFunData(dec_Y_tot$eigenfunctions_tot_funData),1)))
    }
    Y_Z<-lapply(1:n_obs,f)
    Y_Z_funData<-convert_funData_list_to_funData(Y_Z)
    
    f <- function(kk) {
       Reduce("+", Map('*',csi_X[kk, ]%*%PLS , extractObs(multiFunData(dec_Y_tot$eigenfunctions_tot_funData),1)))
      }
    Y_Z_expected<-convert_funData_list_to_funData(lapply(1:n_obs,f))
    
    # Means funData
    M_Xmat<-matrix(0,n_var_X,P)
    M_Ymat<-matrix(0,1,P)
    for (ii in 1:n_var_X) {
      M_Xmat[ii,]<-M_X[[ii]]
    }
    M_Ymat[1,]<-M_Y[[1]]
    M_X_funData<-funData(time_grid,M_Xmat)
    M_Y_funData<-funData(time_grid,M_Ymat)
    
    # Variances funData
    V_Xmat<-matrix(0,n_var_X,P)
    V_Ymat<-matrix(0,1,P)
    for (ii in 1:n_var_X) {
      V_Xmat[ii,]<-V_X[[ii]]
    }
    V_Ymat[1,]<-V_Y[[1]]
    V_X_funData<-funData(time_grid,V_Xmat)
    V_Y_funData<-funData(time_grid,V_Ymat)
  
    Y_Z_fd<-convert_funData_to_fd(Y_Z_funData)
    M_Y_fd<-convert_funData_to_fd(M_Y_funData)
    V_Y_fd<-convert_funData_to_fd(V_Y_funData)
    beta_delta_fd_list_rep<-lapply(1:n_obs, function(ii)beta_delta_fd_sum)
    beta_delta_fd_rep<-convert_fd_list_to_fd(beta_delta_fd_list_rep)
    Y_fd<-(Y_Z_fd+beta_delta_fd_rep)*(V_Y_fd)^(1/2)+M_Y_fd
    Y_funData<-convert_fd_to_funData(Y_fd)
    
    f<-function(ii){
      X_Z_fd_s<-convert_funData_to_fd (X_Z_funData[[ii]])
      M_X_fd<-convert_funData_to_fd(extractObs(M_X_funData,ii))
      V_X_fd<-convert_funData_to_fd(extractObs(V_X_funData,ii))
      (X_Z_fd_s)*(V_X_fd)^(1/2)+M_X_fd
    }
    X_fd <- lapply(1:n_var_X,f)
    
    X_funData<-list()
    for (jj in 1:n_var_X) {
      X_funData[[jj]]<-convert_fd_to_funData(X_fd[[jj]])
    }
    
    XY<-list(X_fd=X_fd,
             Y_fd=Y_fd,
             X_Z=X_Z_funData,
             Y_Z=Y_Z_funData,
             X_funData=X_funData,
             Y_funData=Y_funData,
             csi_X=csi_X,
             csi_Y=csi_Y)
    
    n_var_X<-length(XY$X_fd)
    X_err <- list()
    for (jj in 1:n_var_X) {
      X_err[[jj]]<-matrix(0,n_obs,length(grid_obs))
      for (ii in 1:n_obs) {
        X_err[[jj]][ii,] <-
          eval.fd(grid_obs, XY$X_fd[[jj]][ii])+ mvrnorm(1, rep(0, length(grid_obs)),
                                                        diag(rep(var_x[jj], length(grid_obs))))
        }
    }
    
    X_err_funData <- list()
    for (jj in 1:n_var_X) {
      X_err_funData[[jj]] <-
        funData(grid_obs, matrix(X_err[[jj]], n_obs, length(grid_obs)))
      
    }
    
    Y_err <- matrix(0,n_obs,length(grid_obs))
    for (ii in 1:n_obs) {
      Y_err[ii,] <-
        eval.fd(grid_obs, XY$Y_fd[ii]) + mvrnorm(1, rep(0, length(grid_obs)), diag(rep(var_y[1], length(grid_obs))))
      
    }
   
    Y_err_funData<-funData(grid_obs, matrix(Y_err, n_obs, length(grid_obs)))
    
    XY_err<-list(X_err_funData=X_err_funData,
                 Y_err_funData=Y_err_funData,
                 X_err=X_err,
                 Y_err=Y_err)
    
    # Lists of fd
    basis <- create.bspline.basis(domain,100)
    sm<-c(10^(-4.5),10^(-3.8),10^(-7.3),10^(-2.7))
    x<-grid_obs
    fda_list<-list()
    for (ii in 1:length(XY_err$X_err)) {
      pen <- fdPar(basis,2,sm[ii])
      y<-t(XY_err$X_err[[ii]])
      fda_list[[ii]]<- smooth.basis(x,y,pen)$fd
    }
    pen_y <- fdPar(basis,2,sm[4])
    fda_list[[length(XY_err$X_err)+1]]<-smooth.basis(x,t(XY_err$Y_err),pen_y)$fd    
    
    out<-list(XY=XY,
              XY_err=XY_err,
              fda_list= fda_list,
              Y_Z_expected=Y_Z_expected
    )
    return(out)
    
  }







# Multivariate functional linear regression -------------------------------


 MFLR<-function(Y_MFPCA,X_MFPCA,fda_list,fda_list_z){
  
  n_basis_x <- length(X_MFPCA$scores[1, ])
  n_basis_y <- length(Y_MFPCA$scores[1, ])
  n_var_x<-length(X_MFPCA$functions)
  n_var_y<-length(Y_MFPCA$functions)
  n_obs<-length(X_MFPCA$scores[,1])
  n_var<-n_var_x+n_var_y
  fda_list_FunData_z<-convert_fd_list_to_FunData_list(fda_list_z$fda_list_z)
  multivariate_fda_FunData_z<-multiFunData(fda_list_FunData_z)
  fda_list_FunData<-convert_fd_list_to_FunData_list(fda_list)
  multivariate_fda_FunData<-multiFunData(fda_list_FunData)
  multivariate_fda_FunData_X<-multiFunData(multivariate_fda_FunData[-n_var])
  multivariate_fda_FunData_Y<-multiFunData(multivariate_fda_FunData[n_var])
  multivariate_fda_FunData_X_z<-multiFunData(multivariate_fda_FunData_z[-n_var])
  multivariate_fda_FunData_Y_z<-multiFunData(multivariate_fda_FunData_z[n_var])
  
  # Eigenfunctions
  X_eigenfunctions <- list()
  for (ii in 1:n_basis_x) {
    X_eigenfunctions[[ii]] <- list()
    for (jj in 1:(n_var_x)) {
      X_eigenfunctions[[ii]][[jj]] <-
        extractObs(X_MFPCA$functions[[jj]], ii)
    }
  }
  Y_eigenfunctions <- list()
  for (ii in 1:n_basis_y) {
    Y_eigenfunctions[[ii]] <- list()
    for (jj in 1:(n_var_y)) {
      Y_eigenfunctions[[ii]][[jj]] <-
        extractObs(Y_MFPCA$functions[[jj]], ii)
    }
  }
  
  reg <- lm(Y_MFPCA$scores ~ X_MFPCA$scores-1)
  P <- reg$coefficients
  P<-as.matrix(P)
  
  # Standardize Y_hat 
  y_hat_z <- vector("list", length = n_obs)
  for (kk in 1:n_var_y) {
    y_hat_z[[kk]]<-list()
    for (ii in 1:n_obs) {
      y_scores_hat_cal <-   t(P) %*% X_MFPCA$scores[ii,]
      sum <- 0
      for (jj in 1:n_basis_y) {
        sum <- sum +  Y_eigenfunctions[[jj]][[kk]] * y_scores_hat_cal[jj]
      }
      y_hat_z[[kk]][[ii]] <- sum
    }
  }
  y_hat_z_funData<-convert_funData_list_to_funData(y_hat_z[[1]])
  
  sd_funData <- convert_fd_list_to_FunData_list(fda_list_z$sd_fd)
  mean_funData <- convert_fd_list_to_FunData_list(fda_list_z$mean_fd)
  mean_funData_x <- multiFunData(mean_funData[-n_var])
  mean_funData_y <- multiFunData(mean_funData[n_var])
  sd_funData_x <- multiFunData(sd_funData[-n_var])
  sd_funData_y <- multiFunData(sd_funData[n_var])
  
  # De-standardize
  y_hat <- y_hat_z
  for (kk in 1:n_var_y) {
    for (ii in 1:n_obs) {
      y_hat[[ii]] <-
        mean_funData_y[[kk]] + y_hat_z[[kk]][[ii]] * sd_funData_y[[kk]]
    }
  }
  y_hat_funData<-convert_funData_list_to_funData(y_hat)
  
  # Compute SIGMA_M
  residuals_scores <- reg$residuals
  sigma_M <- (1/n_obs) * crossprod(residuals_scores,residuals_scores)
  
  # Standardize residuals
  res_s <- y_hat_z
  for (ii in 1:n_obs) {
    res_s[[ii]] <-
      extractObs(multivariate_fda_FunData_Y_z, ii)[[1]] - y_hat_z[[1]][[ii]]
  }
  res_s <- convert_funData_list_to_funData(res_s)
  res_fd <- convert_funData_to_fd(res_s)
  mean_res_fd<-mean.fd(res_fd)
  sd_res_fd <- sd.fd(res_fd)
  var_res_fd <- sd_res_fd * sd_res_fd
  var_res_funData <- convert_fd_to_funData(var_res_fd)
  
  # Destandardize residuals
  res <- y_hat
  for (ii in 1:n_obs) {
    res[[ii]] <-
      extractObs(multivariate_fda_FunData_Y, ii)[[1]] - y_hat[[ii]]
  }
  res <- convert_funData_list_to_funData(res)
  
  # Spectral decomposition residuals
  x_multi <- multiFunData(res_s)
  n_basis_res<-10
  Res_MFPCA <-
    MFPCA(x_multi, n_basis_res,  list(list(
      type = "splines1D"
    )))
  
  lambda_sum<-sum(Res_MFPCA$values)
  f<-function (ii){
    sum(Res_MFPCA$values[1:ii])/lambda_sum
  }
  perc<-sapply(1:length(Res_MFPCA$values),f)
  rm(f)
  n_basis_res<-which(perc>0.95)[1]
  Res_MFPCA <-
    MFPCA(x_multi, n_basis_res,  list(list(
      type = "splines1D"
    )))
  
  out <- list(
    reg=reg,
    PLS=P,
    y_hat = y_hat,
    y_hat_z=y_hat_z_funData,
    res_s = res_s,
    mean_funData_x = mean_funData_x,
    mean_funData_y = mean_funData_y,
    sd_funData_x = sd_funData_x,
    sd_funData_y = sd_funData_y,
    Res_MFPCA=Res_MFPCA
  )
  return(out)
} 



# Get T^2 Hotelling and SPE statistics ----------------------------------------------
 
 get_statistics<-function(x,eigenfunctions,sco){
   
   # Compute scores
   if(class(x[[1]])=="funData"){x<-convert_funData_to_fd(x[[1]])}
   if(class(eigenfunctions[[1]])=="funData"){eigenfunctions<-convert_funData_to_fd(eigenfunctions[[1]])
   }
   n_basis_x <- length(eigenfunctions$fdnames$reps)
   n_obs <-  length(x$fdnames$reps)
   scoresmat<-inprod(x,eigenfunctions)
   
   # Alphas 
   alpha_nc<-1-(1-0.05)^(1/2)
   alpha_nn<-1-(1-0.01)^(1/2)
   alpha_rc<-1-(1-0.0027)^(1/2)
   
   # T^2 Hotelling 
   T2hotelling <- rep(0, n_obs)
   var_scores<-var(sco,sco)
   mean_scores<-colMeans(sco)
   if(n_basis_x==1){
     for (ii in 1:n_obs) {
       T2hotelling[ii] <-(scoresmat[ii]-mean_scores) %*% solve(var_scores) %*% (scoresmat[ii]-mean_scores)
     }
   }
   else{
     for (ii in 1:n_obs) {
       T2hotelling[ii] <-t(scoresmat[ii, ]-mean_scores) %*% solve(var_scores) %*% (scoresmat[ii, ]-mean_scores)
     }
   }
   d<-density(T2hotelling,n=2000)
   nc_perc_T2<-quantile(d,(1-alpha_nc),type = 8)
   nn_perc_T2<-quantile(d,(1-alpha_nn),type = 8)
   rc_perc_T2<-quantile(d,(1-alpha_rc),type = 8)
   true_limit<-(n_basis_x*n_obs/(n_obs-n_basis_x+1))*qf(1-alpha_nn,n_basis_x,n_obs-n_basis_x+1)
   
   # SPE 
   eigenfunctions <- convert_fd_to_funData(eigenfunctions)
   x <- convert_fd_to_funData(x)
   if (n_basis_x == 1) {
     f <- function (ii) {
       sum <- 0
       for (jj in 1:n_basis_x) {
         sum <- sum + extractObs(eigenfunctions, jj) * scoresmat[ii]
       }
       x_hat <- sum
       res <- extractObs(x, ii) - x_hat
       res_2 <- res ^ 2
       SPE <- integrate(res_2)
       out <- list(
         x_hat = x_hat,
         res = res,
         res_2 = res_2,
         SPE = SPE
       )
       return(out)
     }
   }
   else{
     f <- function (ii, kk) {
       sum <- 0
       for (jj in 1:n_basis_x) {
         sum <- sum + extractObs(eigenfunctions, jj) * scoresmat[ii, jj]
       }
       x_hat <- sum
       res <- extractObs(x, ii) - x_hat
       res_2 <- res ^ 2
       SPE <- integrate(res_2)
       out <- list(
         x_hat = x_hat,
         res = res,
         res_2 = res_2,
         SPE = SPE
       )
       return(out)
     }
   }
   
   lista<-Map(f,1:n_obs)
   x_hat<-sapply(lista, "[[", 1)
   res<-sapply(lista, "[[", 2)
   res_2<-sapply(lista, "[[", 3)
   SPE<-sapply(lista, "[[", 4)
   res_fun<-convert_funData_list_to_funData(res)
   res2_fun<-convert_funData_list_to_funData(res_2)
   emp_dist_SPE<-density(SPE,n=2000)
   nc_perc_SPE<- quantile(emp_dist_SPE,(1-alpha_nc),type = 8)
   nn_perc_SPE<- quantile(emp_dist_SPE,(1-alpha_nn),type = 8)
   rc_perc_SPE<- quantile(emp_dist_SPE,(1-alpha_rc),type = 8)
   
   out <- list(
     T2hotelling = list(
       values = T2hotelling,
       emp_dist = d,
       nc_perc = nc_perc_T2,
       nn_perc = nn_perc_T2,
       rc_perc = rc_perc_T2
     ),
     SPE = list(
       values = SPE,
       emp_dist = emp_dist_SPE,
       nc_perc = nc_perc_SPE,
       nn_perc = nn_perc_SPE,
       rc_perc = rc_perc_SPE
     ),
     scores = scoresmat,
     res=res_fun
   )
   
   return(out)
 }
 
# Training phase ----------------------------------------------------------

training_phase<-function(fda_list){
  
  n_var <- length(fda_list)
  n_var_x<-n_var-1
  n_var_y<-1
  n_obs <- length(fda_list[[1]]$fdnames$reps)
  fda_list_FunData<-convert_fd_list_to_FunData_list(fda_list)
  multivariate_fda_FunData<-multiFunData(fda_list_FunData)
  
  # Standardize data
  fda_list_z<-standardize_fda_list(fda_list)
  fda_list_FunData_z<-convert_fd_list_to_FunData_list(fda_list_z$fda_list_z)
  multivariate_fda_FunData_z<-multiFunData(fda_list_FunData_z)
  
  # MFPCA
  multivariate_fda_FunData_X<-multiFunData(multivariate_fda_FunData[-n_var])
  multivariate_fda_FunData_Y<-multiFunData(multivariate_fda_FunData[n_var])
  multivariate_fda_FunData_X_z<-multiFunData(multivariate_fda_FunData_z[-n_var])
  multivariate_fda_FunData_Y_z<-multiFunData(multivariate_fda_FunData_z[n_var])
  
  n_basis_x<-30
  n_basis_y<-10
  
  expansion<-list()
  for(ii in 1:(n_var_x)){
    expansion[[ii]]<-list(type = "splines1D")
  }
  
  X_MFPCA <-
    MFPCA(
      multivariate_fda_FunData_X_z,
      n_basis_x,
      expansion
    )
  Y_MFPCA <-
    MFPCA(multivariate_fda_FunData_Y_z, n_basis_y,  list(list(type = "splines1D")))
  
  
  # Choose number of components 
  
  ## X
  lambda_sum<-sum(X_MFPCA$values)
  f<-function (ii){
    sum(X_MFPCA$values[1:ii])/lambda_sum
  }
  perc<-sapply(1:length(X_MFPCA$values),f)
  rm(f)
  n_basis_x<-which(perc>0.95)[1]
  
  ## Y
  lambda_sum<-sum(Y_MFPCA$values)
  f<-function (ii) {
    sum(Y_MFPCA$values[1:ii])/lambda_sum
  }
  perc<-sapply(1:length(Y_MFPCA$values),f)
  rm(f)
  n_basis_y<-which(perc>0.95)[1]
  
  X_MFPCA <-
    MFPCA(
      multivariate_fda_FunData_X_z,
      n_basis_x,
      expansion
    )
  Y_MFPCA <-
    MFPCA(multivariate_fda_FunData_Y_z, n_basis_y, list(list(type = "splines1D")))
  
  # Regression 
  
  cat("Regression")
  REG <- MFLR(Y_MFPCA, X_MFPCA, fda_list, fda_list_z)
  
  
  out<-list(Y_MFPCA=Y_MFPCA,
            X_MFPCA=X_MFPCA,
            REG=REG,
            fda_list_z=fda_list_z)
  return(out)
}



# Tuning phase ------------------------------------------------------------

tuning_phase<-function(fda_list,training){
  
  n_var <- length(fda_list)
  n_obs <- length(fda_list[[1]]$fdnames$reps)
  n_var_x<-n_var-1
  n_var_y<-1
  n_basis_x<-length(training$X_MFPCA$scores[1,])
  n_basis_y<-length(training$Y_MFPCA$scores[1,])
  fda_list_FunData<-convert_fd_list_to_FunData_list(fda_list)
  multivariate_fda_FunData<-multiFunData(fda_list_FunData)
  
  # Standardize data
  fda_list_z <-
    standardize_fda_list_given_means_sds(fda_list, training$fda_list_z$mean_fd, training$fda_list_z$sd_fd)
  fda_list_FunData_z<-convert_fd_list_to_FunData_list(fda_list_z)
  multivariate_fda_FunData_z<-multiFunData(fda_list_FunData_z)
  multivariate_fda_FunData_X<-multiFunData(multivariate_fda_FunData[-n_var])
  multivariate_fda_FunData_Y<-multiFunData(multivariate_fda_FunData[n_var])
  multivariate_fda_FunData_X_z<-multiFunData(multivariate_fda_FunData_z[-n_var])
  multivariate_fda_FunData_Y_z<-multiFunData(multivariate_fda_FunData_z[n_var])
  
  
  # Compute scores
  du<-list()
  for (ii in 1:n_var_x) {
    eig_fd<-convert_funData_to_fd(training$X_MFPCA$functions[[ii]])
    du[[ii]]<-inprod(fda_list_z[[ii]],eig_fd)
  }
  scores<-Reduce("+",du)
  
  y_hat_z <- vector("list", length = n_obs)
  for (ii in 1:n_obs) {
    y_scores_hat_cal <-   t(training$REG$PLS) %*% scores[ii,]
    sum <- 0
    for (jj in 1:n_basis_y) {
      sum <- sum +  extractObs(training$Y_MFPCA$functions, jj)[[1]] * y_scores_hat_cal[jj]
    }
    y_hat_z[[ii]] <- sum
  }
  
  y_hat_z_funData<-convert_funData_list_to_funData(y_hat_z)
  sd_funData <- convert_fd_list_to_FunData_list(training$fda_list_z$sd_fd)
  mean_funData <- convert_fd_list_to_FunData_list(training$fda_list_z$mean_fd)
  mean_funData_x <- multiFunData(mean_funData[-n_var])
  mean_funData_y <- multiFunData(mean_funData[n_var])
  sd_funData_x <- multiFunData(sd_funData[-n_var])
  sd_funData_y <- multiFunData(sd_funData[n_var])
  
  # Destandardize
  y_hat <- y_hat_z
  f <- function(ii) {
    mean_funData_y[[1]] + y_hat_z[[ii]] * sd_funData_y[[1]]
  }
  y_hat<-lapply( 1:n_obs,f)
  y_hat_funData<-convert_funData_list_to_funData(y_hat)
  
  # Compute destandardized residuals
  res_s<-fda_list_z[[4]]-convert_funData_to_fd(convert_funData_list_to_funData( y_hat_z))
  res<-fda_list[[4]]-convert_funData_to_fd(convert_funData_list_to_funData( y_hat))
  statistics_r <-
    get_statistics(res_s,
                   training$REG$Res_MFPCA$functions,
                   training$REG$Res_MFPCA$scores)
  statistics_y <-
    get_statistics(multivariate_fda_FunData_Y_z,
                   training$Y_MFPCA$functions,
                   training$Y_MFPCA$scores)
  out<-list(statistics_r=statistics_r,
            statistics_y=statistics_y,
            res_s=res_s)
  return(out)
}





# Tuning INBA -------------------------------------------------------------

tun_INBA <- function(fda_list) {
  n_obs <- length(fda_list[[4]]$fdnames$reps)
  Y_FunData <- convert_fd_to_funData(fda_list[[4]])
  int <- integrate(Y_FunData)
  d <- density(int, bw = "SJ", n = 2000)
  nn_perc_d <- quantile(d, 1 - (0.01 / 2))
  nn_perc_s <- quantile(d, 0.01 / 2)
  nc_perc_d <- quantile(d, (1 - (0.05 / 2)))
  nc_perc_s <- quantile(d, ((0.05 / 2)))
  
  stand_dev <- sd(int)
  nr <- qnorm(1 - (0.01 / 2), 0, 1)
  media <- mean(int)
  lim_sup <- media + nr * stand_dev
  lim_inf <- media - nr * stand_dev
  out <- list(
    values = int,
    nc_perc_d = nc_perc_d,
    nc_perc_s = nc_perc_s,
    nn_perc_d = nn_perc_d,
    nn_perc_s = nn_perc_s,
    lim_sup = lim_sup,
    lim_inf = lim_inf
  )
  return(out)
}

# Phase II ------------------------------------------------------------------------------

phase_II <- function(fda_list, training, tuning) {
  n_var <- length(fda_list)
  n_obs <- length(fda_list[[1]]$fdnames$reps)
  n_var_x <- n_var - 1
  n_var_y <- 1
  n_basis_x <- length(training$X_MFPCA$scores[1,])
  n_basis_y <- length(training$Y_MFPCA$scores[1,])
  fda_list_FunData <- convert_fd_list_to_FunData_list(fda_list)
  multivariate_fda_FunData <- multiFunData(fda_list_FunData)
  
  # Standardize data
  fda_list_z <-
    standardize_fda_list_given_means_sds(fda_list,
                                         training$fda_list_z$mean_fd,
                                         training$fda_list_z$sd_fd)
  fda_list_FunData_z <- convert_fd_list_to_FunData_list(fda_list_z)
  multivariate_fda_FunData_z <- multiFunData(fda_list_FunData_z)
  multivariate_fda_FunData_X <-
    multiFunData(multivariate_fda_FunData[-n_var])
  multivariate_fda_FunData_Y <-
    multiFunData(multivariate_fda_FunData[n_var])
  multivariate_fda_FunData_X_z <-
    multiFunData(multivariate_fda_FunData_z[-n_var])
  multivariate_fda_FunData_Y_z <-
    multiFunData(multivariate_fda_FunData_z[n_var])
  
  # Compute scores
  du <- list()
  for (ii in 1:n_var_x) {
    eig_fd <- convert_funData_to_fd(training$X_MFPCA$functions[[ii]])
    du[[ii]] <- inprod(fda_list_z[[ii]], eig_fd)
  }
  scores <- Reduce("+", du)
  
  y_hat_z <- vector("list", length = n_obs)
  for (ii in 1:n_obs) {
    y_scores_hat_cal <-   t(training$REG$PLS) %*% scores[ii,]
    sum <- 0
    for (jj in 1:n_basis_y) {
      sum <-
        sum +  extractObs(training$Y_MFPCA$functions, jj)[[1]] * y_scores_hat_cal[jj]
    }
    y_hat_z[[ii]] <- sum
  }
  y_hat_z_funData <- convert_funData_list_to_funData(y_hat_z)
  
  sd_funData <- convert_fd_list_to_FunData_list(training$fda_list_z$sd_fd)
  mean_funData <- convert_fd_list_to_FunData_list(training$fda_list_z$mean_fd)
  mean_funData_x <- multiFunData(mean_funData[-n_var])
  mean_funData_y <- multiFunData(mean_funData[n_var])
  sd_funData_x <- multiFunData(sd_funData[-n_var])
  sd_funData_y <- multiFunData(sd_funData[n_var])
  
  # Destandardize
  y_hat <- y_hat_z
  for (ii in 1:n_obs) {
    y_hat[[ii]] <-
      mean_funData_y[[1]] + y_hat_z[[ii]] * sd_funData_y[[1]]
  }
  
  y_hat_funData <- convert_funData_list_to_funData(y_hat)
  
  # Compute standardized residuals
  res_s <-
    fda_list_z[[4]] - convert_funData_to_fd(convert_funData_list_to_funData(y_hat_z))
  res <-
    fda_list[[4]] - convert_funData_to_fd(convert_funData_list_to_funData(y_hat))
  statistics_r <-
    get_statistics(res_s,
                   training$REG$Res_MFPCA$functions,
                   training$REG$Res_MFPCA$scores)
  statistics_y <-
    get_statistics(
      multivariate_fda_FunData_Y_z,
      training$Y_MFPCA$functions,
      training$Y_MFPCA$scores
    )
  
  # Charts
  lim_res_T2_nn <- tuning$statistics_r$T2hotelling$nn_perc
  lim_res_SPE_nn <- tuning$statistics_r$SPE$nn_perc
  lim_y_T2_nn <- tuning$statistics_y$T2hotelling$nn_perc
  lim_y_SPE_nn <- tuning$statistics_y$SPE$nn_perc
  alpha_res_T2 <-
    (length(which(
      statistics_r$T2hotelling$values >= lim_res_T2_nn
    )) /  n_obs)
  alpha_res_SPE <-
    (length(which(
      statistics_r$SPE$values >= lim_res_SPE_nn
    )) / n_obs)
  alpha_y_T2 <-
    (length(which(
      statistics_y$T2hotelling$values >= lim_y_T2_nn
    )) /  n_obs)
  alpha_y_SPE <-
    (length(which(
      statistics_y$SPE$values >= lim_y_SPE_nn
    )) / n_obs)
  
  ARL_res <- 1 / (1 - (1 - alpha_res_T2) * (1 - alpha_res_SPE))
  ARL_y <- 1 / (1 - (1 - alpha_y_T2) * (1 - alpha_y_SPE))
  
  out <- list(
    ARL_res = ARL_res,
    ARL_y = ARL_y,
    statistics_y = statistics_y,
    statistics_r = statistics_r,
    res_s = res_s,
    res = res
  )
  
  return(out)
}


# Phase II INBA -----------------------------------------------------------

phase_II_INBA<-function(fda_list,tuning_INBA){
  n_obs <- length(fda_list[[4]]$fdnames$reps)
  Y_FunData <- convert_fd_to_funData(fda_list[[4]])
  
  int <- integrate(Y_FunData)
  
  lim_nn_d <- tuning_INBA$nn_perc_d
  lim_nn_s <- tuning_INBA$nn_perc_s
  lim_nc_d <- tuning_INBA$nc_perc_d
  lim_nc_s <- tuning_INBA$nc_perc_s
  alpha_nn <-
    ((length(which(int >= lim_nn_d)) + length(which(int <= lim_nn_s))) /  n_obs)
  alpha_nn_sw<-
    ((length(which(int >= tuning_INBA$lim_sup)) + length(which(int <= tuning_INBA$lim_inf))) /  n_obs)
  
  
  alpha_nc <-
    ((length(which(int >= lim_nc_d)) + length(which(int <= lim_nc_s))) /  n_obs)
  
  ARL_nn <- 1 / alpha_nn
  ARL_nn_sw <- 1 / alpha_nn_sw
  ARL_nc <- 1 / alpha_nc
  out <- list(ARL_nn = ARL_nn,
              ARL_nn_sw=ARL_nn_sw,
              ARL_nc = ARL_nc)
  return(out)
}





