


# Library -----------------------------------------------------------------


library(MFPCA)
library(geoR)
library(limSolve)
library(fda)
library(MASS)
library(spatstat)


# Load functions
source("Functions_simulation.R", encoding="UTF-8")
source("Functions_fd.R", encoding="UTF-8")


# Correlations ------------------------------------------------------------

domain<-c(0,1)
P<-500
time_grid<-seq(domain[1],domain[2] , length.out = P)
TIPI_X<-list("Bessel","gaussian","powered.exponential")
TIPI_Y<-list("Bessel")
kappa_x<-c(0,1,0.5)
kappa_y<-0
n_auto_X<-50
n_auto_Y<-10

CORR_X<-CORR(TIPI_X,P,time_grid,kappa_x)
CORR_Y<-CORR(TIPI_Y,P,time_grid,kappa_y)

dec_X<-Sdecomposition_single(CORR_X,P,time_grid,n_auto_X)
dec_Y<-Sdecomposition_single(CORR_Y,P,time_grid,n_auto_Y)

dec_X_tot<-Sdecomposition_total(dec_X, P, time_grid,n_auto_X)
dec_Y_tot<-Sdecomposition_total(dec_Y, P, time_grid,n_auto_Y)

A<-cbind(diag(dec_X_tot$eigenvalues_tot)[1:10,1:10],diag(rep(1,10)))
b<-dec_Y_tot$eigenvalues_tot
C<-diag(rep(1,20))
d<- rep(0,20)
x<-xranges(A,b,C,d)
a<-sqrt(x[1:10,2])

diag_PLS_1<-(a-0.01); diag_PLS_1[diag_PLS_1<0]<-0 ##R^2=0.9725201
diag_PLS_2<-(a-0.05); diag_PLS_2[diag_PLS_2<0]<-0##R^2=0.8571391
diag_PLS_3<-(a-0.1); diag_PLS_3[diag_PLS_3<0]<-0  #R^2=0.7564374
diag_PLS_l <-
  list(diag_PLS_1,
       diag_PLS_2,
       diag_PLS_3)

# All Shifts ------------------------------------------------------------------

m_f_y_s<-list()
m_f_y_s[[1]]<-list(c(0,30,0),c(0.5,30,0),c(1,30,0),c(1.5,30,0),c(2,30,0))
m_f_y_s[[2]]<-list(c(0,30,0),c(0,30.5,0),c(0,31,0),c(0,31.5,0),c(0,32,0))
m_f_y_s[[3]]<-list(c(0,30,0),c(0,30,0.5),c(0,30,1),c(0,30,1.5),c(0,30,2))
m_f_y_s[[4]]<-list(c(0,30,0),c(0.5,30.5,0),c(1,31,0),c(1.5,31.5,0),c(2,32,0))
m_f_x_1_s<-list(c(-20,20,20),c(-20,20,20.1))
m_f_x_2_s<-list(c(0,4,0),c(0.1,4,0))
m_f_x_3_s<-list(c(-10,14,-8),c(-10,14,-7.5))


# Mean and variance parameters --------------------------------------------

m_medie <-
  list(
    c(15, 20, 50, 70, 100, 130, 170, 180, 190) / 200,
    0,
    c(15, 20, 55, 60, 62, 100, 100, 100, 109, 110, 140, 166, 190) / 200,
    0)
m_deviazioni <- list(c(10, 6, 10, 10, 20, 10, 20, 8, 7) / 200,
                     0,
                     c(10, 6, 4, 4, 7, 1.5, 4, 5, 2, 6, 5, 7, 7) / (200),
                     0)
v_medie<-list(c(15,20,25,30,80,130,170,180,185)/200,
              0,
              c(15,20,30,45,80,105,110,120,125,130,170,180,185)/200,
              0)
v_deviazioni<-list(c(10,12,15,15,15,15,9,9,8)/200,
                   0,
                   c(10,12,10,8,10,7,9,9,8,6,3,2,3)/200,
                   0)
fu<-function(cf
){
  function(x) cf[1]*x^2+cf[2]*x+cf[3]
}


# Shift type definition ---------------------------------------------------

#Y shifts
shift_number<-1
severity_y<-2

#X shifts
severity_x_1<-0
severity_x_2<-0
severity_x_3<-0

diag_PLS<-diag_PLS_l[[1]]
m_f_s_list<-list(m_f_y_s[[shift_number]][[severity_y+1]],m_f_x_1_s[[severity_x_1+1]],m_f_x_2_s[[severity_x_2+1]],m_f_x_3_s[[severity_x_3+1]])
sigma_M_diag<-diag(diag(dec_Y_tot$eigenvalues_tot)-t(rbind(diag(diag_PLS),matrix(0,nrow = 40,ncol=10)))%*%diag(dec_X_tot$eigenvalues_tot)%*%rbind(diag(diag_PLS),matrix(0,nrow = 40,ncol=10)))


# OC means and variances calculation --------------------------------------

domain<-c(0,1)
P<-1000
time_grid<-seq(domain[1],domain[2] , length.out = P)
m_f_y<-m_f_s_list[[1]]
m_f_x_1<-m_f_s_list[[2]]
m_f_x_2<-m_f_s_list[[3]]
m_f_x_3<-m_f_s_list[[4]]



# Means
m_cf<-list(m_f_x_1,
           m_f_x_2,
           m_f_x_3,
           m_f_y)

m_f<-lapply(m_cf,fu)

cost<-list(0.05,0,0.05,0)
# Variances
v_cf<-list(c(0,0,1),
           c(0,0.02,0.3),
           c(-40,150,30),
           c(0,8,1))
v_f<-lapply(v_cf,fu)
cons<-list(0.1,0,2,0)

med<-mapply(me_va,m_medie,m_deviazioni,m_f,cost)
var<-mapply(me_va,v_medie,v_deviazioni,v_f,cost)


V_X<-list(var[,1], var[,2],var[,3])
V_Y<-list(var[,4])
M_X<-list(med[,1],med[,2],med[,3])
M_Y<-list(med[,4])


# IC means and variances calculation --------------------------------------
m_f_y<-c(0,30,0)
m_f_x_1<-c(-20,20,20)
m_f_x_2<-c(0,4,0)
m_f_x_3<-c(-10,14,-8)

# Means
m_cf<-list(m_f_x_1,
           m_f_x_2,
           m_f_x_3,
           m_f_y)

m_f<-lapply(m_cf,fu)

cost<-list(0.05,0,0.05,0)

# Variances
v_cf<-list(c(0,0,1),
           c(0,0.02,0.3),
           c(-40,150,30),
           c(0,8,1))
v_f<-lapply(v_cf,fu)
cons<-list(0.1,0,2,0)

med<-mapply(me_va,m_medie,m_deviazioni,m_f,cost)
var<-mapply(me_va,v_medie,v_deviazioni,v_f,cost)
M_X_c<-list(med[,1], med[,2], med[,3])
M_Y_c<-list(med[,4])

delta_M_X<-mapply(function(x,y){M<-x-y;return(M)},M_X,M_X_c,SIMPLIFY = FALSE)
delta_M_X_Z<-mapply(function(x,y){x*y^(-1/2)},delta_M_X,V_X,SIMPLIFY = FALSE)
time_grid_dec<-seq(0,1,length.out = 500)
delta_M_X_Z_reduced<-mapply(function(x){x[seq(2,length(time_grid),2)]},delta_M_X_Z,SIMPLIFY = FALSE)

PLS <- diag(diag_PLS, n_auto_X, n_auto_Y)
n_var_X<-length(dec_X_tot$fittedcor_tot[[1]])

beta <- list()
for (ii in 1:n_var_X) {
  sum <- 0
  for (jj in 1:n_auto_X) {
    for (kk in 1:n_auto_Y) {
      sum <-
        sum + PLS[jj, kk] * tensorProduct(
          extractObs(dec_X_tot$eigenfunctions_tot_funData[[jj]], ii)
          ,
          extractObs(dec_Y_tot$eigenfunctions_tot_funData[[kk]], 1)
        )
    }
    beta[[ii]] <- sum
    
  }
}

beta_matrix<-mapply(function(a)getX(a)[1,,],beta,SIMPLIFY = FALSE)
beta_matrix_delta<-mapply(function(a,b)rep(1/500,length(a))*(a%*%b),delta_M_X_Z_reduced,
                          beta_matrix,SIMPLIFY = FALSE)
beta_matrix_array<-mapply(
  function(x,y)array(c(x),y),
  beta_matrix_delta,
  list(c(1,length(time_grid_dec)),
       c(1,length(time_grid_dec)),
       c(1,length(time_grid_dec))),
  SIMPLIFY = FALSE)
beta_delta_funData<-mapply(funData,list(seq(0,1,length.out = 500),
                                        seq(0,1,length.out = 500),
                                        seq(0,1,length.out = 500)),
                           beta_matrix_array,
                           SIMPLIFY = FALSE)
beta_delta_fd_list<-mapply(convert_funData_to_fd,beta_delta_funData,
                           SIMPLIFY = FALSE)
beta_delta_fd_sum<-Reduce("+",beta_delta_fd_list)





##################  DATA GENERATION  ###############


# Means and variances Phase I -------------------------------------------------

domain<-c(0,1)
P<-1000
time_grid<-seq(domain[1],domain[2] , length.out = P)

m_f_y<-c(0,30,0)
m_f_x_1<-c(-20,20,20)
m_f_x_2<-c(0,4,0)
m_f_x_3<-c(-10,14,-8)


# Means
m_cf<-list(m_f_x_1,
           m_f_x_2,
           m_f_x_3,
           m_f_y)

m_f<-lapply(m_cf,fu)

cost<-list(0.05,0,0.05,0)

# Variances
v_cf<-list(c(0,0,1),
           c(0,0.02,0.3),
           c(-40,150,30),
           c(0,8,1))
v_f<-lapply(v_cf,fu)
cons<-list(0.1,0,2,0)

med<-mapply(me_va,m_medie,m_deviazioni,m_f,cost)
var<-mapply(me_va,v_medie,v_deviazioni,v_f,cost)


# Data generation Phase I -------------------------------------------------

n_auto_X<-50
n_auto_Y<-10
V_X<-list(var[,1], var[,2],var[,3])
V_Y<-list(var[,4])
M_X<-list(med[,1],med[,2],med[,3])
M_Y<-list(med[,4])
grid_obs <- seq(domain[1],domain[2] , length.out = 150) 
var_x <- (c(0.3, 0.05, 0.3))^2 
var_y <- c(0.3)^2



beta_delta_fd_sum_0<-beta_delta_fd_sum-beta_delta_fd_sum

# Data for training
n_obs<-1000
run_training <- single_simulation_run_shift (
  dec_X_tot,
  dec_Y_tot,
  n_auto_X,
  n_auto_Y,
  diag_PLS,
  n_obs,
  sigma_M_diag,
  V_X,
  V_Y,
  M_X,
  M_Y,
  grid_obs,
  var_x,
  var_y,
  time_grid,
  P,
  beta_delta_fd_sum_0)


# Data for tuning
n_obs<-3000
run_tuning<- single_simulation_run_shift (
  dec_X_tot,
  dec_Y_tot,
  n_auto_X,
  n_auto_Y,
  diag_PLS,
  n_obs,
  sigma_M_diag,
  V_X,
  V_Y,
  M_X,
  M_Y,
  grid_obs,
  var_x,
  var_y,
  time_grid,
  P,
  beta_delta_fd_sum_0
)

# Means and variances Phase II -------------------------------------------------

m_f_y<-m_f_s_list[[1]]
m_f_x_1<-m_f_s_list[[2]]
m_f_x_2<-m_f_s_list[[3]]
m_f_x_3<-m_f_s_list[[4]]

# Means
m_cf<-list(m_f_x_1,
           m_f_x_2,
           m_f_x_3,
           m_f_y)

m_f<-lapply(m_cf,fu)

cost<-list(0.05,0,0.05,0)

# Variances
v_medie<-list(c(15,20,25,30,80,130,170,180,185)/200,
              0,
              c(15,20,30,45,80,105,110,120,125,130,170,180,185)/200,
              0)
v_deviazioni<-list(c(10,12,15,15,15,15,9,9,8)/200,
                   0,
                   c(10,12,10,8,10,7,9,9,8,6,3,2,3)/200,
                   0)
v_cf<-list(c(0,0,1),
           c(0,0.02,0.3),
           c(-40,150,30),
           c(0,8,1))
v_f<-lapply(v_cf,fu)
cons<-list(0.1,0,2,0)

med<-mapply(me_va,m_medie,m_deviazioni,m_f,cost)
var<-mapply(me_va,v_medie,v_deviazioni,v_f,cost)

V_X<-list(var[,1], var[,2],var[,3])
V_Y<-list(var[,4])
M_X<-list(med[,1],med[,2],med[,3])
M_Y<-list(med[,4])


# Data generation for Phase II ------------------------------------------------------------------

n_obs<-4000

run_phase_II<- single_simulation_run_shift (
  dec_X_tot,
  dec_Y_tot,
  n_auto_X,
  n_auto_Y,
  diag_PLS,
  n_obs,
  sigma_M_diag,
  V_X,
  V_Y,
  M_X,
  M_Y,
  grid_obs,
  var_x,
  var_y,
  time_grid,
  P,
  beta_delta_fd_sum
)


################## PHASE I/II ##################


# Phase I -----------------------------------------------------------------

# Training
training <- training_phase(run_training$fda_list)

# Tuning 
tuning <- tuning_phase(run_tuning$fda_list, training)
tuning_INBA<-tun_INBA(run_tuning$fda_list)

# Phase II---------------------------------------------------------------

fase_2 <- phase_II(run_phase_II$fda_list, training, tuning)
fase_2_INBA<-phase_II_INBA(run_phase_II$fda_list,tuning_INBA)

ARLS<-c(fase_2$ARL_res, fase_2$ARL_y,fase_2_INBA$ARL_nn,fase_2_INBA$ARL_nn_sw)

